#define _GNU_SOURCE 

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <pci/pci.h>
#include "libldetect.h"
#include "libldetect-private.h"
#include "common.h"

static int pci_find_modules(struct pciusb_entries entries, int no_subid) {
  return pciusb_find_modules(entries, "pcitable", no_subid);
}

extern struct pciusb_entries pci_probe(int probe_type) {
  FILE *dev;
  char buf[512];
  unsigned short devbusfn;
  unsigned int id;
  struct pciusb_entry t[100];
  struct pciusb_entries r;

  if (!(dev = fopen("/proc/bus/pci/devices", "r"))) exit(1);

  for (r.nb = 0; fgets(buf, sizeof(buf) - 1, dev) && r.nb < psizeof(t); r.nb++) {
    struct pciusb_entry e;
    pciusb_initialize(&e);

    sscanf(buf, "%hx %x", &devbusfn, &id);
    e.vendor = id >> 16;
    e.device = id & 0xffff;
    e.pci_bus = devbusfn >> 8;
    e.pci_device = (devbusfn & 0xff) >> 3;
    e.pci_function = (devbusfn & 0xff) & 0x07;
    {
      char file[25];
      int f;
      snprintf(file, sizeof(file), "/proc/bus/pci/%02x/%02x.%d", e.pci_bus, e.pci_device, e.pci_function);
      if (probe_type == 1 && ((f = open(file, O_RDONLY)) != -1)) {
        if (pread(f, &e.class, 2, 10) < 0) 
	  		fprintf(stderr, "Can't pread() from device %02x/%02x.%d\n",
				e.pci_bus, e.pci_device, e.pci_function);
        if (pread(f, &e.subvendor, 2, 0x2c) < 0) 
	  		fprintf(stderr,"Can't pread() from device %02x/%02x.%d\n",
				e.pci_bus,e.pci_device, e.pci_function);
        if (pread(f, &e.subdevice, 2, 0x2e) < 0) 
	  		fprintf(stderr,"Can't pread() from device %02x/%02x.%d\n",
				e.pci_bus,e.pci_device, e.pci_function);

	if ((e.subvendor == 0 && e.subdevice == 0) ||
	    (e.subvendor == e.vendor && e.subdevice == e.device)) {
	  e.subvendor = 0xffff;
	  e.subdevice = 0xffff;
	}
	  unsigned char class_prog = 0;
	  if (pread(f, &class_prog, 1, 9) < 0) {
 		fprintf(stderr,"Can't pread() from device %02x/%02x.%d\n",
		e.pci_bus,e.pci_device, e.pci_function);
	 }
	close(f);
      }
    }
    t[r.nb] = e;
  }
  fclose(dev);
  r.entries = memdup(t, sizeof(struct pciusb_entry) * r.nb);

  if (pci_find_modules(r, probe_type)) return r;

  /* ok, let's try again with subids */
  if (probe_type == 0) return pci_probe(1);

  /* should not happen */
  exit(1);
}

